##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: zcml.py 78325 2007-07-25 15:43:51Z fdrake $
"""
__docformat__ = "reStructuredText"

import os

from zope import interface
from zope import component
from zope import schema

from zope.component import zcml

from zope.configuration.exceptions import ConfigurationError
import zope.configuration.fields

from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.pagetemplate.interfaces import IPageTemplate
from zope.configuration.fields import GlobalObject
from zope import schema

from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile

from z3c.viewtemplate.macro import Macro

from zope.i18nmessageid import MessageFactory
_ = MessageFactory('zope')


class ITemplateDirective(interface.Interface):
    """Parameters for the template directive."""

    template = zope.configuration.fields.Path(
            title=_("Content-generating template."),
            description=_("Refers to a file containing a page template (should "
                          "end in extension ``.pt`` or ``.html``)."),
            required=False,
            )

    macro = schema.TextLine(
            title = _(u'Macro'),
            description = _(u"""
                The macro to be used.
                This allows us to define different macros in on template.
                The template designer can now create macros for the whole site
                in a single page template, and ViewTemplate can then extract
                the macros for single viewlets or views.
                If no macro is given the whole template is used for rendering.
                """),
            required = False,
            default = u'',
            )

    for_ = GlobalObject(
            title = _(u'View'),
            description = _(u'The view for which the template should be used'),
            required = False,
            default=interface.Interface,
            )

    layer = GlobalObject(
            title = _(u'Layer'),
            description = _(u'The layer for which the template should be used'),
            required = False,
            default=IDefaultBrowserLayer,
            )

    contentType = schema.BytesLine(
        title = _(u'Content Type'),
        description=_(u'The content type identifies the type of data.'),
        default='text/html',
        required=False,
        )



class TemplateFactory(object):

    template = None

    def __init__(self, filename, macro, contentType):
        self.filename = filename
        self.macro = macro
        self.contentType = contentType

    def __call__(self, view, request):
        if self.template is None:
            self.template= ViewPageTemplateFile(self.filename,
                                                content_type=self.contentType)
        if self.macro is None:
            return self.template
        return Macro(self.template, self.macro, view,
                     request, self.contentType)


def templateDirective(_context,
                      template,
                      macro=None,
                      for_=interface.Interface,
                      layer=IDefaultBrowserLayer,
                      contentType='text/html',
                      ):
    # Make sure that the template exists
    template = os.path.abspath(str(_context.path(template)))
    if not os.path.isfile(template):
        raise ConfigurationError("No such file", template)

    factory = TemplateFactory(template, macro, contentType)

    # register the template
    zcml.adapter(_context, (factory,), IPageTemplate, (for_, layer))




##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: macro.py 74237 2007-04-18 14:55:25Z jukart $
"""
__docformat__ = "reStructuredText"

from StringIO import StringIO

from zope.tal.talinterpreter import TALInterpreter

class Macro(object):
    """Provides a single macro from a template for rendering."""

    def __init__(self, template, macroName, view, request, contentType):
        self.template = template
        self.macroName = macroName
        self.view = view
        self.request = request
        self.contentType = contentType

    def __call__(self, *args, **kwargs):
        try:
            program = self.template.macros[self.macroName]
        except TypeError:
            raise KeyError('Macro "%s" not found in file "%s"'% (
                self.macroName, self.template.filename))
        output = StringIO(u'')
        namespace = self.template.pt_getContext(self.view, self.request)
        context = self.template.pt_getEngineContext(namespace)
        TALInterpreter(program, None,
                       context, output, tal=True, showtal=False,
                       strictinsert=0, sourceAnnotations=False)()
        if not self.request.response.getHeader("Content-Type"):
            self.request.response.setHeader("Content-Type",
                                            self.contentType)
        return output.getvalue()




##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: interfaces.py 70224 2006-09-19 10:12:53Z jukart $
"""
__docformat__ = "reStructuredText"

from zope.contentprovider.interfaces import IContentProvider


class ITemplatedContentProvider(IContentProvider):
    """Content providers implementing this interface use templates to generate
    their content.
    """



#!python
from setuptools import setup, find_packages

setup(name='z3c.viewtemplate',
      version='0.3.0',
      author = "Zope Community",
      author_email = "zope3-dev@zope.org",
      description = open("README.txt").read(),
      license = "ZPL 2.1",
      keywords = "view template zope zope3",
      url='http://svn.zope.org/z3c.viewtemplate',

      zip_safe=False,
      packages=find_packages('src'),
      include_package_data=True,
      package_dir = {'':'src'},
      namespace_packages=['z3c',],
      extras_require = dict(test=['zope.app.testing',
                                  'zope.testing',
                                  ]),
      install_requires=[
          'setuptools',
          'zope.app.pagetemplate',
          'zope.component',
          'zope.configuration',
          'zope.contentprovider',
          'zope.i18nmessageid',
          'zope.pagetemplate',
          'zope.publisher',
          'zope.tal',
          ],
      )




##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: baseview.py 80223 2007-09-27 14:15:32Z dobe $
"""
__docformat__ = "reStructuredText"

from zope import interface
from zope import component
from zope import event

from zope.pagetemplate.interfaces import IPageTemplate
from zope.publisher.browser import BrowserView

from z3c.viewtemplate.interfaces import ITemplatedContentProvider
from zope.contentprovider.interfaces import BeforeUpdateEvent

class TemplatedContentProvider(object):
    interface.implements(ITemplatedContentProvider)

    template = None

    def update(self):
        pass

    def render(self):
        if self.template is None:
            template = component.getMultiAdapter(
                    (self, self.request), IPageTemplate)
            return template(self)
        return self.template()


class BaseView(TemplatedContentProvider, BrowserView):

    def __call__(self):
        event.notify(BeforeUpdateEvent(self, self.request))
        self.update()
        return self.render()

